using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Web.Services3.Design;
using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Security.Tokens;
using System.Security.Cryptography.X509Certificates;
using Microsoft.Web.Services3.Security;
using System.Xml;
using System.Collections;

namespace ConexionPlatino
{
    class PlatinoSecurityAssertion : SecurityPolicyAssertion
    {
        String sSerialNumber;
        String sUri;
        String sPassword;
        bool bValid = false;
        SignatureOptions sigOptions;
        string[] NombreParametrosPlatino;
        string[] ValorParametrosPlatino;

        // Guardamos las variables de configuracin
        public PlatinoSecurityAssertion(String SerialNumber, String Uri, String Password, SignatureOptions nsigOption, string[] sNombreParametrosPlatino,string[] sValorParametrosPlatino)
            : base()
        {

            this.sSerialNumber = SerialNumber;
            this.sUri = Uri;
            this.sPassword = Password;
            this.sigOptions = nsigOption;
            if (sNombreParametrosPlatino != null && sValorParametrosPlatino != null && sValorParametrosPlatino.Length == sNombreParametrosPlatino.Length)
            {
                this.NombreParametrosPlatino = sNombreParametrosPlatino;
                this.ValorParametrosPlatino = sValorParametrosPlatino;
                
            }

        }


        // Sobrescribimos la funcion de Output
        public override SoapFilter CreateClientOutputFilter(FilterCreationContext context)
        {
            PlatinoSecurityAssertionClientOutputFilter filter = new PlatinoSecurityAssertionClientOutputFilter(this, this.sSerialNumber, this.sUri, this.sPassword, this.sigOptions,this.NombreParametrosPlatino,this.ValorParametrosPlatino);

            if (filter.IsValid()) bValid = true;
            return filter;

        }

        public override SoapFilter CreateClientInputFilter(FilterCreationContext context)
        {            
            return null;            

        }

        public override SoapFilter CreateServiceInputFilter(FilterCreationContext context)
        {

            return null; // new CustomSecurityServerInputFilter(this);

        }

        public override SoapFilter CreateServiceOutputFilter(FilterCreationContext context)
        {

            return null; //new CustomSecurityServerOutputFilter(this);

        }

       

    }
    // Generamos la clase de OutputFilter
    class PlatinoSecurityAssertionClientOutputFilter: SendSecurityFilter
    {
        X509SecurityToken clientToken = null;
        UsernameToken userToken = null;
        SignatureOptions sigOptions;
        string[] NombreParametrosPlatino;
        string[] ValorParametrosPlatino;

        // Variables de creacin, creaos usertoken y BinarysecurityTokken
        public PlatinoSecurityAssertionClientOutputFilter(PlatinoSecurityAssertion parentAssertion, String SerialNumber, String Uri, String Password, SignatureOptions nsigOptions,string[] sNombreParametrosPlatino,string[] sValorParametrosPlatino)
            : base(parentAssertion.ServiceActor, true)
        {
            // Obtenga el token de seguridad del Cliente.
            X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly);
            X509Certificate2Collection colCert = store.Certificates.Find(X509FindType.FindBySerialNumber, SerialNumber, false);

            if (colCert.Count <= 0) return;

            X509Certificate2 clientCert = colCert[0];
            clientToken = new X509SecurityToken(clientCert);
            userToken = new UsernameToken(Uri, Password, PasswordOption.SendPlainText);
            this.sigOptions = nsigOptions;

            this.NombreParametrosPlatino = sNombreParametrosPlatino;
            this.ValorParametrosPlatino = sValorParametrosPlatino;
        }

        // funcin es valid
        public bool IsValid()            
        {
            // Obtenga el token de seguridad del Cliente.
            return ( clientToken != null && userToken != null);

        }

        // Sobrescribimos sel Secure Mensaje
        public override void SecureMessage(SoapEnvelope envelope, Security security)
        {
                        
            // Quitamos el address
            XmlNode actionNode = envelope.Header["wsa:Action"];
            envelope.Header.RemoveChild(actionNode);

            // Quitamos el MessageID
            XmlNode messageNode = envelope.Header["wsa:MessageID"];
            envelope.Header.RemoveChild(messageNode);

            // Quitamos el ReplyTo
            XmlNode replyToNode = envelope.Header["wsa:ReplyTo"];
            envelope.Header.RemoveChild(replyToNode);


            // Quitamos el To
            XmlNode toNode = envelope.Header["wsa:To"];
            envelope.Header.RemoveChild(toNode);

            
            // Firme el mensaje SOAP con el token de seguridad del cliente.
            security.Actor = "http://www.gobiernodecanarias.org/Platino/Authentication/1.0";

            security.Tokens.Add(clientToken);
            security.Tokens.Add(userToken);
           
            // Buscamos los tokens, es bastante complicado porque son propiedades privadas
            System.Reflection.FieldInfo FI = security.Tokens.GetType().GetField("_tokens",
                System.Reflection.BindingFlags.Public |
                System.Reflection.BindingFlags.NonPublic |
                System.Reflection.BindingFlags.Instance);

            Hashtable tokens = FI == null ? null : (Hashtable)FI.GetValue(security.Tokens);

            string idUserToken = "";

            foreach (DictionaryEntry o in tokens)
            {
                if (o.Value.ToString().Contains("UsernameToken"))
                {
                    idUserToken = o.Key.ToString();
                    break;
                }


            }


            // Aadimos platino y cabeceras
            XmlNode nodeplatino = null;
            if (NombreParametrosPlatino != null && NombreParametrosPlatino.Length > 0)
            {
                XmlElement platino = envelope.CreateElement("platino", "platinoHeaders", "http://platino.gobcan.es");
                XmlAttribute attrib = envelope.CreateAttribute("xmlns", "wsu", "http://www.w3.org/2000/xmlns/");
                XmlAttribute attrib2 = envelope.CreateAttribute("wsu", "Id", "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd");

                attrib.Value = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd";
                platino.Attributes.Append(attrib);

                attrib2.Value = "id-platinoHeader";
                platino.Attributes.Append(attrib2);

                nodeplatino = envelope.Header.InsertAfter(platino, envelope.Header.LastChild);

                // Creamos los datos de platino
                for (short i = 0; i < NombreParametrosPlatino.Length; i++)
                {
                    XmlElement platinoparameter = envelope.CreateElement("platino", "parameter", "http://platino.gobcan.es");
                    platinoparameter.InnerText = ValorParametrosPlatino[i];
                    XmlAttribute attribparameter = envelope.CreateAttribute("name");
                    attribparameter.Value = NombreParametrosPlatino[i];
                    platinoparameter.Attributes.Append(attribparameter);
                    nodeplatino.InsertAfter(platinoparameter, nodeplatino.LastChild);

                }
            }        

            // Firmamos
            MessageSignature objMessageSignature = new MessageSignature(clientToken);

            // Firmamos UserToken
            if (!String.IsNullOrEmpty(idUserToken))
            {
                SignatureReference sig = new SignatureReference("#" + idUserToken);
                sig.AddTransform(new Microsoft.Web.Services3.Security.Xml.XmlDsigExcC14NTransform());
                objMessageSignature.AddReference(sig);
            }

            // Firmamos Platino
            if (nodeplatino != null)
            {
                SignatureReference sig = new SignatureReference("#id-platinoHeader");
                sig.AddTransform(new Microsoft.Web.Services3.Security.Xml.XmlDsigExcC14NTransform());

                objMessageSignature.AddReference(sig);
            }

            objMessageSignature.SignatureOptions = this.sigOptions;                       
            security.Elements.Add(objMessageSignature);
            
        }
    }


   
}